@@ -32,20 +32,21 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdbool.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/mman.h>
 
-#define CACHE_VERSION   0x0c
+#define CACHE_VERSION   0x0d
 
 #define CACHE_64_BIT	0x01
 #define CACHE_BE	0x02
 
 #define CACHE_RESERVED_PATTERN  	0xff
 
-#define CACHE_ENTRY_USED_SIZE		24
-#define CACHE_ENTRY_RESERVED_SIZE	56
+#define CACHE_ENTRY_USED_SIZE		28
+#define CACHE_ENTRY_RESERVED_SIZE	52
 #define CACHE_ENTRY_TOTAL_SIZE	(CACHE_ENTRY_RESERVED_SIZE + CACHE_ENTRY_USED_SIZE)
 
 // Cmus Track Cache version X + 4 bytes flags
@@ -61,6 +62,7 @@ struct cache_entry {
 	int64_t mtime;
 	int32_t duration;
 	int32_t bitrate;
+	int32_t bpm;
 
 	// when introducing new fields decrease the reserved space accordingly
 	uint8_t _reserved[CACHE_ENTRY_RESERVED_SIZE];
@@ -81,7 +83,7 @@ static struct track_info *hash_table[HASH_SIZE];
 static char *cache_filename;
 static int total;
 
-pthread_mutex_t cache_mutex = CMUS_MUTEX_INITIALIZER;
+struct fifo_mutex cache_mutex = FIFO_MUTEX_INITIALIZER;
 
 
 static void add_ti(struct track_info *ti, unsigned int hash)
@@ -131,6 +133,7 @@ static struct track_info *cache_entry_to_ti(struct cache_entry *e)
 	ti->bitrate = e->bitrate;
 	ti->mtime = e->mtime;
 	ti->play_count = e->play_count;
+	ti->bpm = e->bpm;
 
 	// count strings (filename + codec + codec_profile + key/val pairs)
 	count = 0;
@@ -207,7 +210,7 @@ void cache_remove_ti(struct track_info *ti)
 static int read_cache(void)
 {
 	unsigned int size, offset = 0;
-	struct stat st;
+	struct stat st = {};
 	char *buf;
 	int fd;
 
@@ -217,7 +220,7 @@ static int read_cache(void)
 			return 0;
 		return -1;
 	}
-	fstat(fd,  &st);
+	fstat(fd, &st);
 	if (st.st_size < sizeof(cache_header))
 		goto close;
 	size = st.st_size;
@@ -284,7 +287,7 @@ static int ti_filename_cmp(const void *a, const void *b)
 	return strcmp(ai->filename, bi->filename);
 }
 
-static struct track_info **get_track_infos(void)
+static struct track_info **get_track_infos(bool reference)
 {
 	struct track_info **tis;
 	int i, c;
@@ -295,6 +298,8 @@ static struct track_info **get_track_infos(void)
 		struct track_info *ti = hash_table[i];
 
 		while (ti) {
+			if (reference)
+				track_info_ref(ti);
 			tis[c++] = ti;
 			ti = ti->next;
 		}
@@ -328,6 +333,7 @@ static void write_ti(int fd, struct gbuf *buf, struct track_info *ti, unsigned i
 	e.bitrate = ti->bitrate;
 	e.mtime = ti->mtime;
 	e.play_count = ti->play_count;
+	e.bpm = ti->bpm;
 	len[count] = strlen(ti->filename) + 1;
 	e.size += len[count++];
 	len[count] = (ti->codec ? strlen(ti->codec) : 0) + 1;
@@ -380,7 +386,7 @@ int cache_close(void)
 		return -1;
 	}
 
-	tis = get_track_infos();
+	tis = get_track_infos(false);
 
 	gbuf_grow(&buf, 64 * 1024 - 1);
 	gbuf_add_bytes(&buf, cache_header, sizeof(cache_header));
@@ -462,7 +468,7 @@ struct track_info *cache_get_ti(const char *filename, int force)
 
 struct track_info **cache_refresh(int *count, int force)
 {
-	struct track_info **tis = get_track_infos();
+	struct track_info **tis = get_track_infos(true);
 	int i, n = total;
 
 	for (i = 0; i < n; i++) {
@@ -471,6 +477,8 @@ struct track_info **cache_refresh(int *count, int force)
 		struct stat st;
 		int rc = 0;
 
+		cache_yield();
+
 		/*
 		 * If no-one else has reference to tis[i] then it is set to NULL
 		 * otherwise:
@@ -484,13 +492,13 @@ struct track_info **cache_refresh(int *count, int force)
 			rc = stat(ti->filename, &st);
 			if (!rc && !force && ti->mtime == st.st_mtime) {
 				// unchanged
+				track_info_unref(ti);
 				tis[i] = NULL;
 				continue;
 			}
 		}
 
 		hash = hash_str(ti->filename);
-		track_info_ref(ti);
 		do_cache_remove_ti(ti, hash);
 
 		if (!rc) {
@@ -498,7 +506,7 @@ struct track_info **cache_refresh(int *count, int force)
 			struct track_info *new_ti;
 
 			// clear cache-only entries
-			if (force && ti->ref == 1) {
+			if (force && track_info_unique_ref(ti)) {
 				track_info_unref(ti);
 				tis[i] = NULL;
 				continue;
@@ -508,7 +516,7 @@ struct track_info **cache_refresh(int *count, int force)
 			if (new_ti) {
 				add_ti(new_ti, hash);
 
-				if (ti->ref == 1) {
+				if (track_info_unique_ref(ti)) {
 					track_info_unref(ti);
 					tis[i] = NULL;
 				} else {
@@ -521,7 +529,7 @@ struct track_info **cache_refresh(int *count, int force)
 		}
 
 		// deleted
-		if (ti->ref == 1) {
+		if (track_info_unique_ref(ti)) {
 			track_info_unref(ti);
 			tis[i] = NULL;
 		} else {
