@@ -21,12 +21,6 @@
 
 #include <stdio.h>
 
-/*
- * warning: this header does not contain include guards!
- */
-#include <libcue/libcue.h>
-
-
 char *associated_cue(const char *filename);
 int cue_get_ntracks(const char *filename);
 char *construct_cue_url(const char *cue_filename, int track_n);
