@@ -16,6 +16,7 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "utils.h"
 #include "job.h"
 #include "worker.h"
 #include "cache.h"
@@ -34,51 +35,138 @@
 #include "discid.h"
 #include "xstrjoin.h"
 #include "ui_curses.h"
-#ifdef HAVE_CONFIG
-#include "config/cue.h"
-#endif
-#ifdef CONFIG_CUE
 #include "cue_utils.h"
-#endif
 
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <sys/mman.h>
 
-static struct track_info *ti_buffer[32];
-static int ti_buffer_fill;
+enum job_result_var {
+	JOB_RES_ADD,
+	JOB_RES_UPDATE,
+	JOB_RES_UPDATE_CACHE,
+	JOB_RES_PL_DELETE,
+};
+
+enum update_kind {
+	UPDATE_NONE = 0,
+	UPDATE_REMOVE = 1,
+	UPDATE_MTIME_CHANGED = 2,
+};
+
+struct job_result {
+	struct list_head node;
+
+	enum job_result_var var;
+	union {
+		struct {
+			add_ti_cb add_cb;
+			size_t add_num;
+			struct track_info **add_ti;
+			void *add_opaque;
+		};
+		struct {
+			size_t update_num;
+			struct track_info **update_ti;
+			enum update_kind *update_kind;
+		};
+		struct {
+			size_t update_cache_num;
+			struct track_info **update_cache_ti;
+		};
+		struct {
+			void (*pl_delete_cb)(struct playlist *);
+			struct playlist *pl_delete_pl;
+		};
+	};
+};
+
+int job_fd;
+static int job_fd_priv;
+
+static LIST_HEAD(job_result_head);
+static pthread_mutex_t job_mutex = CMUS_MUTEX_INITIALIZER;
+
+#define TI_CAP 32
+static struct track_info **ti_buffer;
+static size_t ti_buffer_fill;
 static struct add_data *jd;
 
+#define job_lock() cmus_mutex_lock(&job_mutex)
+#define job_unlock() cmus_mutex_unlock(&job_mutex)
+
+void job_init(void)
+{
+	init_pipes(&job_fd, &job_fd_priv);
+
+	worker_init();
+}
+
+void job_exit(void)
+{
+	worker_remove_jobs_by_type(JOB_TYPE_ANY);
+	worker_exit();
+
+	close(job_fd);
+	close(job_fd_priv);
+}
+
+static void job_push_result(struct job_result *res)
+{
+	job_lock();
+	list_add_tail(&res->node, &job_result_head);
+	job_unlock();
+
+	notify_via_pipe(job_fd_priv);
+}
+
+static struct job_result *job_pop_result(void)
+{
+	struct job_result *res = NULL;
+
+	job_lock();
+	if (!list_empty(&job_result_head)) {
+		struct list_head *item = job_result_head.next;
+		list_del(item);
+		res = container_of(item, struct job_result, node);
+	}
+	job_unlock();
+
+	return res;
+}
+
 static void flush_ti_buffer(void)
 {
-	int i;
+	struct job_result *res = xnew(struct job_result, 1);
+
+	res->var = JOB_RES_ADD;
+	res->add_cb = jd->add;
+	res->add_num = ti_buffer_fill;
+	res->add_ti = ti_buffer;
+	res->add_opaque = jd->opaque;
+
+	job_push_result(res);
 
-	editable_lock();
-	for (i = 0; i < ti_buffer_fill; i++) {
-		jd->add(ti_buffer[i]);
-		track_info_unref(ti_buffer[i]);
-	}
-	editable_unlock();
 	ti_buffer_fill = 0;
+	ti_buffer = NULL;
 }
 
 static void add_ti(struct track_info *ti)
 {
-	if (ti_buffer_fill == N_ELEMENTS(ti_buffer))
+	if (ti_buffer_fill == TI_CAP)
 		flush_ti_buffer();
+	if (!ti_buffer)
+		ti_buffer = xnew(struct track_info *, TI_CAP);
 	ti_buffer[ti_buffer_fill++] = ti;
 }
 
-#ifdef CONFIG_CUE
 static int add_file_cue(const char *filename);
-#endif
 
 static void add_file(const char *filename, int force)
 {
 	struct track_info *ti;
 
-#ifdef CONFIG_CUE
 	if (!is_cue_url(filename)) {
 		if (force || lookup_cache_entry(filename, hash_str(filename)) == NULL) {
 			int done = add_file_cue(filename);
@@ -86,7 +174,6 @@ static void add_file(const char *filename, int force)
 				return;
 		}
 	}
-#endif
 
 	cache_lock();
 	ti = cache_get_ti(filename, force);
@@ -96,7 +183,6 @@ static void add_file(const char *filename, int force)
 		add_ti(ti);
 }
 
-#ifdef CONFIG_CUE
 static int add_file_cue(const char *filename)
 {
 	int n_tracks;
@@ -122,7 +208,6 @@ static int add_file_cue(const char *filename)
 	free(cue_filename);
 	return 1;
 }
-#endif
 
 static void add_url(const char *url)
 {
@@ -272,7 +357,8 @@ static int handle_line(void *data, const char *line)
 static void add_pl(const char *filename)
 {
 	char *buf;
-	int size, reverse;
+	ssize_t size;
+	int reverse;
 
 	buf = mmap_file(filename, &size);
 	if (size == -1)
@@ -289,7 +375,7 @@ static void add_pl(const char *filename)
 	}
 }
 
-void do_add_job(void *data)
+static void do_add_job(void *data)
 {
 	jd = data;
 	switch (jd->type) {
@@ -311,82 +397,155 @@ void do_add_job(void *data)
 	case FILE_TYPE_INVALID:
 		break;
 	}
-	if (ti_buffer_fill)
+	if (ti_buffer)
 		flush_ti_buffer();
 	jd = NULL;
 }
 
-void free_add_job(void *data)
+static void free_add_job(void *data)
 {
 	struct add_data *d = data;
 	free(d->name);
 	free(d);
-	ui_curses_notify();
 }
 
-void do_update_job(void *data)
+static void job_handle_add_result(struct job_result *res)
+{
+	for (size_t i = 0; i < res->add_num; i++) {
+		res->add_cb(res->add_ti[i], res->add_opaque);
+		track_info_unref(res->add_ti[i]);
+	}
+
+	free(res->add_ti);
+}
+
+void job_schedule_add(int type, struct add_data *data)
+{
+	worker_add_job(type | JOB_TYPE_ADD, do_add_job, free_add_job, data);
+}
+
+static void do_update_job(void *data)
 {
 	struct update_data *d = data;
 	int i;
+	enum update_kind *kind = xnew(enum update_kind, d->used);
+	struct job_result *res;
 
 	for (i = 0; i < d->used; i++) {
 		struct track_info *ti = d->ti[i];
 		struct stat s;
 		int rc;
 
-		/* stat follows symlinks, lstat does not */
 		rc = stat(ti->filename, &s);
 		if (rc || d->force || ti->mtime != s.st_mtime || ti->duration == 0) {
-			int force = ti->duration == 0;
-			editable_lock();
-			lib_remove(ti);
-			editable_unlock();
-
-			cache_lock();
-			cache_remove_ti(ti);
-			cache_unlock();
-
-			if (!is_cue_url(ti->filename) && !is_http_url(ti->filename) && rc) {
-				d_print("removing dead file %s\n", ti->filename);
-			} else {
-				if (ti->mtime != s.st_mtime)
-					d_print("mtime changed: %s\n", ti->filename);
-				cmus_add(lib_add_track, ti->filename, FILE_TYPE_FILE, JOB_TYPE_LIB, force);
-			}
+			kind[i] = UPDATE_NONE;
+			if (!is_cue_url(ti->filename) && !is_http_url(ti->filename) && rc)
+				kind[i] |= UPDATE_REMOVE;
+			else if (ti->mtime != s.st_mtime)
+				kind[i] |= UPDATE_MTIME_CHANGED;
+		} else {
+			track_info_unref(ti);
+			d->ti[i] = NULL;
 		}
-		track_info_unref(ti);
 	}
+
+	res = xnew(struct job_result, 1);
+
+	res->var = JOB_RES_UPDATE;
+	res->update_num = d->used;
+	res->update_ti = d->ti;
+	res->update_kind = kind;
+
+	job_push_result(res);
+
+	d->ti = NULL;
 }
 
-void free_update_job(void *data)
+static void free_update_job(void *data)
 {
 	struct update_data *d = data;
 
-	free(d->ti);
+	if (d->ti) {
+		for (size_t i = 0; i < d->used; i++)
+			track_info_unref(d->ti[i]);
+		free(d->ti);
+	}
 	free(d);
-	ui_curses_notify();
 }
 
-void do_update_cache_job(void *data)
+static void job_handle_update_result(struct job_result *res)
+{
+	for (size_t i = 0; i < res->update_num; i++) {
+		struct track_info *ti = res->update_ti[i];
+		int force;
+
+		if (!ti)
+			continue;
+
+		lib_remove(ti);
+
+		cache_lock();
+		cache_remove_ti(ti);
+		cache_unlock();
+
+		if (res->update_kind[i] & UPDATE_REMOVE) {
+			d_print("removing dead file %s\n", ti->filename);
+		} else {
+			if (res->update_kind[i] & UPDATE_MTIME_CHANGED)
+				d_print("mtime changed: %s\n", ti->filename);
+			force = ti->duration == 0;
+			cmus_add(lib_add_track, ti->filename, FILE_TYPE_FILE,
+					JOB_TYPE_LIB, force, NULL);
+		}
+
+		track_info_unref(ti);
+	}
+
+	free(res->update_kind);
+	free(res->update_ti);
+}
+
+void job_schedule_update(struct update_data *data)
+{
+	worker_add_job(JOB_TYPE_LIB | JOB_TYPE_UPDATE, do_update_job,
+			free_update_job, data);
+}
+
+static void do_update_cache_job(void *data)
 {
 	struct update_cache_data *d = data;
+	int count;
 	struct track_info **tis;
-	int i, count;
+	struct job_result *res;
 
 	cache_lock();
 	tis = cache_refresh(&count, d->force);
-	editable_lock();
-	player_info_lock();
-	for (i = 0; i < count; i++) {
-		struct track_info *new, *old = tis[i];
+	cache_unlock();
+
+	res = xnew(struct job_result, 1);
+	res->var = JOB_RES_UPDATE_CACHE;
+	res->update_cache_ti = tis;
+	res->update_cache_num = count;
+	job_push_result(res);
+}
+
+static void free_update_cache_job(void *data)
+{
+	free(data);
+}
+
+static void job_handle_update_cache_result(struct job_result *res)
+{
+	for (size_t i = 0; i < res->update_cache_num; i++) {
+		struct track_info *new, *old = res->update_cache_ti[i];
 
 		if (!old)
 			continue;
 
 		new = old->next;
 		if (lib_remove(old) && new)
-			lib_add_track(new);
-		editable_update_track(&pl_editable, old, new);
+			lib_add_track(new, NULL);
+		pl_update_track(old, new);
 		editable_update_track(&pq_editable, old, new);
 		if (player_info.ti == old && new) {
 			track_info_ref(new);
@@ -397,14 +556,72 @@ void do_update_cache_job(void *data)
 		if (new)
 			track_info_unref(new);
 	}
-	player_info_unlock();
-	editable_unlock();
-	cache_unlock();
-	free(tis);
+	free(res->update_cache_ti);
 }
 
-void free_update_cache_job(void *data)
+void job_schedule_update_cache(int type, struct update_cache_data *data)
 {
-	free(data);
-	ui_curses_notify();
+	worker_add_job(type | JOB_TYPE_UPDATE_CACHE, do_update_cache_job,
+			free_update_cache_job, data);
+}
+
+static void do_pl_delete_job(void *data)
+{
+	/*
+	 * If PL jobs are canceled this function won't run. Hence we push the
+	 * result in the free function.
+	 */
+}
+
+static void free_pl_delete_job(void *data)
+{
+	struct pl_delete_data *pdd = data;
+	struct job_result *res;
+
+	res = xnew(struct job_result, 1);
+	res->var = JOB_RES_PL_DELETE;
+	res->pl_delete_cb = pdd->cb;
+	res->pl_delete_pl = pdd->pl;
+	job_push_result(res);
+
+	free(pdd);
+}
+
+static void job_handle_pl_delete_result(struct job_result *res)
+{
+	res->pl_delete_cb(res->pl_delete_pl);
+}
+
+void job_schedule_pl_delete(struct pl_delete_data *data)
+{
+	worker_add_job(JOB_TYPE_PL | JOB_TYPE_DELETE, do_pl_delete_job,
+			free_pl_delete_job, data);
+}
+
+static void job_handle_result(struct job_result *res)
+{
+	switch (res->var) {
+	case JOB_RES_ADD:
+		job_handle_add_result(res);
+		break;
+	case JOB_RES_UPDATE:
+		job_handle_update_result(res);
+		break;
+	case JOB_RES_UPDATE_CACHE:
+		job_handle_update_cache_result(res);
+		break;
+	case JOB_RES_PL_DELETE:
+		job_handle_pl_delete_result(res);
+		break;
+	}
+	free(res);
+}
+
+void job_handle(void)
+{
+	clear_pipe(job_fd, -1);
+
+	struct job_result *res;
+	while ((res = job_pop_result()))
+		job_handle_result(res);
 }
