@@ -29,6 +29,7 @@
 #include "options.h"
 #include "mpris.h"
 #include "cmus.h"
+#include "lib.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -354,7 +355,9 @@ static void update_rg_scale(void)
 	if (!player_info_priv.ti || !replaygain)
 		return;
 
-	if (replaygain == RG_TRACK || replaygain == RG_TRACK_PREFERRED) {
+	bool avoid_album_gain = replaygain == RG_SMART && (!play_library || shuffle == SHUFFLE_TRACKS || cmus_queue_active());
+	
+	if (replaygain == RG_TRACK || replaygain == RG_TRACK_PREFERRED || avoid_album_gain) {
 		gain = player_info_priv.ti->rg_track_gain;
 		peak = player_info_priv.ti->rg_track_peak;
 	} else {
@@ -363,7 +366,7 @@ static void update_rg_scale(void)
 	}
 
 	if (isnan(gain)) {
-		if (replaygain == RG_TRACK_PREFERRED) {
+		if (replaygain == RG_TRACK_PREFERRED || avoid_album_gain) {
 			gain = player_info_priv.ti->rg_album_gain;
 			peak = player_info_priv.ti->rg_album_peak;
 		} else if (replaygain == RG_ALBUM_PREFERRED) {
