@@ -107,10 +107,18 @@ int using_utf8 = 0;
 static char *lib_autosave_filename;
 static char *play_queue_autosave_filename;
 
+#define PRINT_BUFFER_SZ 4096
+/* one character can take up to 4 bytes in UTF-8 */
+#define PRINT_BUFFER_MAX_WIDTH (PRINT_BUFFER_SZ / 4 - 1)
+
+static char print_buffer[PRINT_BUFFER_SZ];
+/* destination buffer for utf8_encode_to_buf and utf8_decode */
+static char conv_buffer[PRINT_BUFFER_SZ];
+
 /* shown error message and time stamp
  * error is cleared if it is older than 3s and key was pressed
  */
-static char error_buf[512];
+static char error_buf[PRINT_BUFFER_SZ];
 static time_t error_time = 0;
 /* info messages are displayed in different color */
 static int msg_is_error;
@@ -118,14 +126,6 @@ static int error_count = 0;
 
 static char *server_address = NULL;
 
-static char print_buffer[1024];
-
-/* destination buffer for utf8_encode_to_buf and utf8_decode */
-static char conv_buffer[512];
-
-/* one character can take up to 4 bytes in UTF-8 */
-#define print_buffer_max_width (sizeof(print_buffer) / 4 - 1)
-
 /* used for messages to the client */
 static int client_fd = -1;
 
@@ -139,9 +139,9 @@ static int tree_win_w = 0;
 static int track_win_x = 0;
 static int track_win_w = 0;
 
-static int editable_win_x = 0;
-static int editable_win_w = 0;
-static int editable_active = 1;
+static int win_x = 0;
+static int win_w = 0;
+static int win_active = 1;
 
 static int show_cursor;
 static int cursor_x;
@@ -467,45 +467,17 @@ static void dump_print_buffer(int row, int col)
  */
 static int format_str(char *buf, const char *str, int width)
 {
-	int s = 0, ellipsis_pos = 0, cut_double_width = 0;
-	size_t d = 0;
+	int w = width;
+	size_t copy_bytes = u_copy_chars(buf, str, &w);
 
-	while (1) {
-		uchar u;
-		int w;
-
-		u = u_get_char(str, &s);
-		if (u == 0) {
-			memset(buf + d, ' ', width);
-			d += width;
-			break;
-		}
-
-		w = u_char_width(u);
-		if (width == 3)
-			ellipsis_pos = d;
-		if (width == 4 && w == 2) {
-			/* can't cut double-width char */
-			ellipsis_pos = d + 1;
-			cut_double_width = 1;
-		}
-
-		width -= w;
-		if (width < 0) {
-			/* does not fit */
-			d = ellipsis_pos;
-			if (cut_double_width) {
-				/* first half of the double-width char */
-				buf[d - 1] = ' ';
-			}
-			buf[d++] = '.';
-			buf[d++] = '.';
-			buf[d++] = '.';
-			break;
-		}
-		u_set_char(buf, &d, u);
+	if (w > 0) {
+		memset(buf + copy_bytes, ' ', w);
+		copy_bytes += w;
+	} else if (str[copy_bytes] != '\0') {
+		copy_bytes = mark_clipped_text(buf, width);
 	}
-	return d;
+	buf[copy_bytes] = '\0';
+	return copy_bytes;
 }
 
 static void sprint(int row, int col, const char *str, int width)
@@ -519,28 +491,6 @@ static void sprint(int row, int col, const char *str, int width)
 	dump_print_buffer(row, col);
 }
 
-static void sprint_ascii(int row, int col, const char *str, int len)
-{
-	int l;
-
-	l = strlen(str);
-	len -= 2;
-
-	print_buffer[0] = ' ';
-	if (l > len) {
-		memcpy(print_buffer + 1, str, len - 3);
-		print_buffer[len - 2] = '.';
-		print_buffer[len - 1] = '.';
-		print_buffer[len - 0] = '.';
-	} else {
-		memcpy(print_buffer + 1, str, l);
-		memset(print_buffer + 1 + l, ' ', len - l);
-	}
-	print_buffer[len + 1] = ' ';
-	print_buffer[len + 2] = 0;
-	(void) mvaddstr(row, col, print_buffer);
-}
-
 static inline void fopt_set_str(struct format_option *fopt, const char *str)
 {
 	BUG_ON(fopt->type != FO_STR);
@@ -667,7 +617,7 @@ const struct format_option *get_global_fopts(void)
 	static const char *cont_strs[] = { " ", "C" };
 	static const char *follow_strs[] = { " ", "F" };
 	static const char *repeat_strs[] = { " ", "R" };
-	static const char *shuffle_strs[] = { " ", "S" };
+	static const char *shuffle_strs[] = { " ", "S", "&" };
 	int buffer_fill, vol, vol_left, vol_right;
 	int duration = -1;
 
@@ -719,7 +669,7 @@ static void print_tree(struct window *win, int row, struct iter *iter)
 	struct artist *artist;
 	struct album *album;
 	struct iter sel;
-	int current, selected, active, pos;
+	int current, selected, active;
 
 	artist = iter_to_artist(iter);
 	album = iter_to_album(iter);
@@ -744,14 +694,11 @@ static void print_tree(struct window *win, int row, struct iter *iter)
 	print_buffer[0] = ' ';
 	if (album) {
 		fill_track_fopts_album(album);
-		format_print(print_buffer + 1, tree_win_w - 2, tree_win_format, track_fopts);
+		format_print(print_buffer + 1, tree_win_w - 1, tree_win_format, track_fopts);
 	} else {
 		fill_track_fopts_artist(artist);
-		format_print(print_buffer + 1, tree_win_w - 2, tree_win_artist_format, track_fopts);
+		format_print(print_buffer + 1, tree_win_w - 1, tree_win_artist_format, track_fopts);
 	}
-	pos = strlen(print_buffer);
-	print_buffer[pos++] = ' ';
-	print_buffer[pos++] = 0;
 	dump_print_buffer(row + 1, tree_win_x);
 }
 
@@ -779,7 +726,7 @@ static void print_track(struct window *win, int row, struct iter *iter)
 		dump_print_buffer(row + 1, track_win_x);
 
 		bkgdset(pairs[CURSED_SEPARATOR]);
-		for(pos = track_win_x + len.llen; pos < COLS - len.rlen; ++pos)
+		for(pos = track_win_x + len.llen + len.mlen; pos < win_w - len.rlen; ++pos)
 			(void) mvaddch(row + 1, pos, ACS_HLINE);
 
 		return;
@@ -826,12 +773,12 @@ static void print_editable(struct window *win, int row, struct iter *iter)
 	selected = iters_equal(iter, &sel);
 
 	if (selected) {
-		cursor_x = editable_win_x;
+		cursor_x = win_x;
 		cursor_y = 1 + row;
 	}
 
-	active = editable_active;
-	if (!selected && track->marked) {
+	active = win_active;
+	if (!selected && !!track->marked) {
 		selected = 1;
 		active = 0;
 	}
@@ -847,8 +794,8 @@ static void print_editable(struct window *win, int row, struct iter *iter)
 	} else if (*list_win_alt_format) {
 		format = list_win_alt_format;
 	}
-	format_print(print_buffer, editable_win_w, format, track_fopts);
-	dump_print_buffer(row + 1, editable_win_x);
+	format_print(print_buffer, win_w, format, track_fopts);
+	dump_print_buffer(row + 1, win_x);
 }
 
 static void print_browser(struct window *win, int row, struct iter *iter)
@@ -878,12 +825,7 @@ static void print_browser(struct window *win, int row, struct iter *iter)
 		cursor_y = 1 + row;
 	}
 
-	/* file name encoding == terminal encoding. no need to convert */
-	if (using_utf8) {
-		sprint(row + 1, 0, e->name, COLS);
-	} else {
-		sprint_ascii(row + 1, 0, e->name, COLS);
-	}
+	sprint(row + 1, 0, e->name, win_w);
 }
 
 static void print_filter(struct window *win, int row, struct iter *iter)
@@ -925,7 +867,7 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 	}
 
 	snprintf(buf, sizeof(buf), "%c%c%c%-15s  %.235s", ch1, ch2, ch3, e->name, e_filter);
-	pos = format_str(print_buffer, buf, COLS - 1);
+	pos = format_str(print_buffer, buf, win_w - 1);
 	print_buffer[pos++] = ' ';
 	print_buffer[pos] = 0;
 	dump_print_buffer(row + 1, 0);
@@ -970,7 +912,7 @@ static void print_help(struct window *win, int row, struct iter *iter)
 		opt->get(opt->data, buf + len, sizeof(buf) - len);
 		break;
 	}
-	pos = format_str(print_buffer, buf, COLS - 1);
+	pos = format_str(print_buffer, buf, win_w - 1);
 	print_buffer[pos++] = ' ';
 	print_buffer[pos] = 0;
 	dump_print_buffer(row + 1, 0);
@@ -981,17 +923,14 @@ static void update_window(struct window *win, int x, int y, int w, const char *t
 {
 	struct iter iter;
 	int nr_rows;
-	int c, i;
+	int i;
 
 	win->changed = 0;
 
 	bkgdset(pairs[CURSED_WIN_TITLE]);
-	c = snprintf(print_buffer, w + 1, " %s", title);
-	if (c > w)
-		c = w;
-	memset(print_buffer + c, ' ', w - c + 1);
-	print_buffer[w] = 0;
+	snprintf(print_buffer, sizeof(print_buffer), " %s ", title);
 	dump_print_buffer(y, x);
+
 	nr_rows = window_get_nr_rows(win);
 	i = 0;
 	if (window_get_top(win, &iter)) {
@@ -1014,17 +953,18 @@ static void update_window(struct window *win, int x, int y, int w, const char *t
 
 static void update_tree_window(void)
 {
-	update_window(lib_tree_win, tree_win_x, 0, tree_win_w,
-			"Artist / Album", print_tree);
+	char title[PRINT_BUFFER_SZ];
+	format_str(title, "Artist / Album", tree_win_w - 1);
+	update_window(lib_tree_win, tree_win_x, 0, tree_win_w, title, print_tree);
 }
 
 static void update_track_window(void)
 {
-	char title[512];
+	char title[PRINT_BUFFER_SZ];
 
 	/* it doesn't matter what format options we use because the format
 	 * string does not contain any format charaters */
-	format_print(title, track_win_w - 2, "Track%=Library", track_fopts);
+	format_print(title, track_win_w - 2, "Track%= Library", track_fopts);
 	update_window(lib_track_win, track_win_x, 0, track_win_w, title,
 			print_track);
 }
@@ -1043,7 +983,7 @@ static void print_pl_list(struct window *win, int row, struct iter *iter)
 	size_t prefix_w = strlen(prefix);
 	format_str(print_buffer, prefix, prefix_w);
 
-	if (tree_win_w >= prefix_w)
+	if (tree_win_w > prefix_w)
 		format_str(print_buffer + prefix_w, info.name,
 				tree_win_w - prefix_w);
 
@@ -1052,33 +992,35 @@ static void print_pl_list(struct window *win, int row, struct iter *iter)
 
 static void update_pl_list(struct window *win)
 {
-	update_window(win, tree_win_x, 0, tree_win_w, "Playlist",
-			print_pl_list);
+	char title[PRINT_BUFFER_SZ];
+	format_str(title, "Playlist", tree_win_w - 1);
+	update_window(win, tree_win_x, 0, tree_win_w, title, print_pl_list);
 }
 
 static void update_pl_tracks(struct window *win)
 {
-	char title[512];
+	char title[PRINT_BUFFER_SZ];
+	int win_w_tmp = win_w;
 
-	editable_win_x = track_win_x;
-	editable_win_w = track_win_w;
-	editable_active = pl_get_cursor_in_track_window();
+	win_x = track_win_x;
+	win_w = track_win_w;
+	win_active = pl_get_cursor_in_track_window();
 
 	get_global_fopts();
 	fopt_set_time(&track_fopts[TF_TOTAL], pl_visible_total_time(), 0);
 
-	format_print(title, track_win_w - 2, "Track%=%{total}", track_fopts);
+	format_print(title, track_win_w - 2, "Track%= %{total}", track_fopts);
 	update_window(win, track_win_x, 0, track_win_w, title, print_editable);
 
-	editable_active = 1;
-	editable_win_x = 0;
-	editable_win_w = COLS;
+	win_active = 1;
+	win_x = 0;
+	win_w = win_w_tmp;
 }
 
 static const char *pretty(const char *path)
 {
 	static int home_len = -1;
-	static char buf[256];
+	static char buf[PRINT_BUFFER_SZ];
 
 	if (home_len == -1)
 		home_len = strlen(home_dir);
@@ -1095,7 +1037,7 @@ static const char * const sorted_names[2] = { "", "sorted by " };
 
 static void update_editable_window(struct editable *e, const char *title, const char *filename)
 {
-	char buf[512];
+	char buf[PRINT_BUFFER_SZ];
 	int pos;
 
 	if (filename) {
@@ -1119,7 +1061,8 @@ static void update_editable_window(struct editable *e, const char *title, const
 			sorted_names[e->shared->sort_str[0] != 0],
 			e->shared->sort_str);
 
-	update_window(e->shared->win, 0, 0, COLS, buf, &print_editable);
+	format_str(buf, buf, win_w - 2);
+	update_window(e->shared->win, 0, 0, win_w, buf, &print_editable);
 }
 
 static void update_sorted_window(void)
@@ -1136,7 +1079,7 @@ static void update_play_queue_window(void)
 
 static void update_browser_window(void)
 {
-	char title[512];
+	char title[PRINT_BUFFER_SZ];
 	char *dirname;
 
 	if (using_utf8) {
@@ -1147,17 +1090,22 @@ static void update_browser_window(void)
 		dirname = conv_buffer;
 	}
 	snprintf(title, sizeof(title), "Browser - %.501s", dirname);
-	update_window(browser_win, 0, 0, COLS, title, print_browser);
+	format_str(title, title, win_w - 2);
+	update_window(browser_win, 0, 0, win_w, title, print_browser);
 }
 
 static void update_filters_window(void)
 {
-	update_window(filters_win, 0, 0, COLS, "Library Filters", print_filter);
+	char title[PRINT_BUFFER_SZ];
+	format_str(title, "Library Filters", win_w - 2);
+	update_window(filters_win, 0, 0, win_w, title, print_filter);
 }
 
 static void update_help_window(void)
 {
-	update_window(help_win, 0, 0, COLS, "Settings", print_help);
+	char title[PRINT_BUFFER_SZ];
+	format_str(title, "Settings", win_w - 2);
+	update_window(help_win, 0, 0, win_w, title, print_help);
 }
 
 static void draw_separator(void)
@@ -1216,7 +1164,7 @@ static void do_update_view(int full)
 
 static void do_update_statusline(void)
 {
-	format_print(print_buffer, COLS, statusline_format, get_global_fopts());
+	format_print(print_buffer, win_w, statusline_format, get_global_fopts());
 	bkgdset(pairs[CURSED_STATUSLINE]);
 	dump_print_buffer(LINES - 2, 0);
 
@@ -1283,7 +1231,7 @@ static void do_update_commandline(void)
 	if (input_mode == SEARCH_MODE)
 		ch = search_direction == SEARCH_FORWARD ? '/' : '?';
 
-	if (w <= COLS - 2) {
+	if (w <= win_w - 2) {
 		addch(ch);
 		idx = u_copy_chars(print_buffer, str, &w);
 		print_buffer[idx] = 0;
@@ -1296,17 +1244,17 @@ static void do_update_commandline(void)
 		/* cursor pos (width, not chars. doesn't count the ':') */
 		cw = u_str_nwidth(str, cmdline.cpos);
 
-		skip = cw + 2 - COLS;
+		skip = cw + 2 - win_w;
 		if (skip > 0) {
 			/* skip the ':' */
 			skip--;
 
 			/* skip rest (if any) */
-			idx = u_skip_chars(str, &skip);
+			idx = u_skip_chars(str, &skip, true);
 
-			width = COLS;
+			width = win_w;
 			idx = u_copy_chars(print_buffer, str + idx, &width);
-			while (width < COLS) {
+			while (width > 0) {
 				/* cursor is at end of the buffer
 				 * print 1, 2 or 3 spaces
 				 *
@@ -1319,14 +1267,14 @@ static void do_update_commandline(void)
 				 * to print 3 spaces.
 				 */
 				print_buffer[idx++] = ' ';
-				width++;
+				width--;
 			}
 			print_buffer[idx] = 0;
 			dump_buffer(print_buffer);
 		} else {
-			/* print ':' + COLS - 1 chars */
+			/* print ':' + win_w - 1 chars */
 			addch(ch);
-			width = COLS - 1;
+			width = win_w - 1;
 			idx = u_copy_chars(print_buffer, str, &width);
 			print_buffer[idx] = 0;
 			dump_buffer(print_buffer);
@@ -1371,18 +1319,18 @@ static void do_update_titleline(void)
 		}
 
 		if (use_alt_format && *current_alt_format) {
-			format_print(print_buffer, COLS, current_alt_format, track_fopts);
+			format_print(print_buffer, win_w, current_alt_format, track_fopts);
 		} else {
-			format_print(print_buffer, COLS, current_format, track_fopts);
+			format_print(print_buffer, win_w, current_format, track_fopts);
 		}
 		dump_print_buffer(LINES - 3, 0);
 
 		/* set window title */
 		if (use_alt_format && *window_title_alt_format) {
-			format_print(print_buffer, print_buffer_max_width,
+			format_print(print_buffer, PRINT_BUFFER_MAX_WIDTH,
 					window_title_alt_format, track_fopts);
 		} else {
-			format_print(print_buffer,  print_buffer_max_width,
+			format_print(print_buffer,  PRINT_BUFFER_MAX_WIDTH,
 					window_title_format, track_fopts);
 		}
 
@@ -1423,7 +1371,7 @@ static int cmdline_cursor_column(void)
 	/* width of the text in the buffer before cursor */
 	cw = u_str_nwidth(str, cmdline.cpos);
 
-	if (1 + cw < COLS) {
+	if (1 + cw < win_w) {
 		/* whole line is visible */
 		return 1 + cw;
 	}
@@ -1432,21 +1380,17 @@ static int cmdline_cursor_column(void)
 
 	/* check if the first visible char in cmdline would be halved
 	 * double-width character (or invalid byte <xx>) which is not possible.
-	 * we need to skip the whole character and move cursor to COLS - 2
+	 * we need to skip the whole character and move cursor to win_w - 2
 	 * column. */
-	skip = cw + 2 - COLS;
+	skip = cw + 2 - win_w;
 
 	/* skip the ':' */
 	skip--;
 
 	/* skip rest */
 	s = skip;
-	u_skip_chars(str, &s);
-	if (s > skip) {
-		/* the last skipped char was double-width or <xx> */
-		return COLS - 1 - (s - skip);
-	}
-	return COLS - 1;
+	u_skip_chars(str, &s, true);
+	return win_w - 1 + s;
 }
 
 static void post_update(void)
@@ -1546,12 +1490,12 @@ void update_filterline(void)
 	if (cur_view != TREE_VIEW && cur_view != SORTED_VIEW)
 		return;
 	if (lib_live_filter) {
-		char buf[512];
+		char buf[PRINT_BUFFER_SZ];
 		int w;
 		bkgdset(pairs[CURSED_STATUSLINE]);
 		snprintf(buf, sizeof(buf), "filtered: %s", lib_live_filter);
-		w = clamp(strlen(buf) + 2, COLS/4, COLS/2);
-		sprint(LINES-4, COLS-w, buf, w);
+		w = clamp(strlen(buf) + 2, win_w/4, win_w/2);
+		sprint(LINES-4, win_w-w, buf, w);
 	}
 }
 
@@ -1602,7 +1546,7 @@ void error_msg(const char *format, ...)
 
 enum ui_query_answer yes_no_query(const char *format, ...)
 {
-	char buffer[512];
+	char buffer[PRINT_BUFFER_SZ];
 	va_list ap;
 	int ret = 0;
 
@@ -1899,7 +1843,7 @@ static void sig_shutdown(int sig)
 	cmus_running = 0;
 }
 
-static volatile sig_atomic_t needs_to_resize = 1;
+static volatile sig_atomic_t needs_to_resize = 0;
 
 static void sig_winch(int sig)
 {
@@ -1926,11 +1870,13 @@ static void resize_tree_view(int w, int h)
 	tree_win_w = w * ((float)tree_width_percent / 100.0f);
 	if (tree_width_max && tree_win_w > tree_width_max)
 		tree_win_w = tree_width_max;
+	/* at least one character of formatted text and one space either side */
+	if (tree_win_w < 3)
+		tree_win_w = 3;
 	track_win_w = w - tree_win_w - 1;
-	if (tree_win_w < 8)
-		tree_win_w = 8;
-	if (track_win_w < 8)
-		track_win_w = 8;
+	if (track_win_w < 3)
+		track_win_w = 3;
+
 	tree_win_x = 0;
 	track_win_x = tree_win_w + 1;
 
@@ -1939,6 +1885,37 @@ static void resize_tree_view(int w, int h)
 	window_set_nr_rows(lib_track_win, h);
 }
 
+static void update_window_size(void)
+{
+	int w, h;
+	int columns, lines;
+
+	if (get_window_size(&lines, &columns) == 0) {
+		needs_to_resize = 0;
+#if HAVE_RESIZETERM
+		resizeterm(lines, columns);
+#endif
+		w = COLS;
+		if (w > PRINT_BUFFER_MAX_WIDTH)
+			w = PRINT_BUFFER_MAX_WIDTH;
+		h = LINES - 3;
+		if (w < 3)
+			w = 3;
+		if (h < 2)
+			h = 2;
+		win_w = w;
+		resize_tree_view(w, h);
+		window_set_nr_rows(lib_editable.shared->win, h - 1);
+		pl_set_nr_rows(h - 1);
+		window_set_nr_rows(pq_editable.shared->win, h - 1);
+		window_set_nr_rows(filters_win, h - 1);
+		window_set_nr_rows(help_win, h - 1);
+		window_set_nr_rows(browser_win, h - 1);
+	}
+	clearok(curscr, TRUE);
+	refresh();
+}
+
 static void update(void)
 {
 	int needs_view_update = 0;
@@ -1948,34 +1925,10 @@ static void update(void)
 	int needs_spawn = 0;
 
 	if (needs_to_resize) {
-		int w, h;
-		int columns, lines;
-
-		if (get_window_size(&lines, &columns) == 0) {
-			needs_to_resize = 0;
-#if HAVE_RESIZETERM
-			resizeterm(lines, columns);
-#endif
-			editable_win_w = COLS;
-			w = COLS;
-			h = LINES - 3;
-			if (w < 16)
-				w = 16;
-			if (h < 2)
-				h = 2;
-			resize_tree_view(w, h);
-			window_set_nr_rows(lib_editable.shared->win, h - 1);
-			pl_set_nr_rows(h - 1);
-			window_set_nr_rows(pq_editable.shared->win, h - 1);
-			window_set_nr_rows(filters_win, h - 1);
-			window_set_nr_rows(help_win, h - 1);
-			window_set_nr_rows(browser_win, h - 1);
-			needs_title_update = 1;
-			needs_status_update = 1;
-			needs_command_update = 1;
-		}
-		clearok(curscr, TRUE);
-		refresh();
+		update_window_size();
+		needs_title_update = 1;
+		needs_status_update = 1;
+		needs_command_update = 1;
 	}
 
 	if (player_info.status_changed)
@@ -2185,8 +2138,9 @@ static void main_loop(void)
 		fd_set set;
 		struct timeval tv;
 		int poll_mixer = 0;
-		int i, nr_fds = 0;
-		int fds[NR_MIXER_FDS];
+		int i;
+		int nr_fds_vol = 0, fds_vol[NR_MIXER_FDS];
+		int nr_fds_out = 0, fds_out[NR_MIXER_FDS];
 		struct list_head *item;
 		struct client *client;
 
@@ -2221,18 +2175,24 @@ static void main_loop(void)
 			SELECT_ADD_FD(client->fd);
 		}
 		if (!soft_vol) {
-			nr_fds = mixer_get_fds(fds);
-			if (nr_fds <= 0) {
+			nr_fds_vol = mixer_get_fds(MIXER_FDS_VOLUME, fds_vol);
+			if (nr_fds_vol <= 0) {
 				poll_mixer = 1;
 				if (!tv.tv_usec)
 					tv.tv_usec = 500e3;
 			}
-			for (i = 0; i < nr_fds; i++) {
-				BUG_ON(fds[i] <= 0);
-				SELECT_ADD_FD(fds[i]);
+			for (i = 0; i < nr_fds_vol; i++) {
+				BUG_ON(fds_vol[i] <= 0);
+				SELECT_ADD_FD(fds_vol[i]);
 			}
 		}
 
+		nr_fds_out = mixer_get_fds(MIXER_FDS_OUTPUT, fds_out);
+		for (i = 0; i < nr_fds_out; i++) {
+			BUG_ON(fds_out[i] <= 0);
+			SELECT_ADD_FD(fds_out[i]);
+		}
+
 		rc = select(fd_high + 1, &set, NULL, NULL, tv.tv_usec ? &tv : NULL);
 		if (poll_mixer) {
 			int ol = volume_l;
@@ -2254,14 +2214,24 @@ static void main_loop(void)
 			continue;
 		}
 
-		for (i = 0; i < nr_fds; i++) {
-			if (FD_ISSET(fds[i], &set)) {
+		for (i = 0; i < nr_fds_vol; i++) {
+			if (FD_ISSET(fds_vol[i], &set)) {
 				d_print("vol changed\n");
 				mixer_read_volume();
 				mpris_volume_changed();
 				update_statusline();
 			}
 		}
+		for (i = 0; i < nr_fds_out; i++) {
+			if (FD_ISSET(fds_out[i], &set)) {
+				d_print("out changed\n");
+				if (pause_on_output_change) {
+					player_pause_playback();
+					update_statusline();
+				}
+				clear_pipe(fds_out[i], -1);
+			}
+		}
 		if (FD_ISSET(server_socket, &set))
 			server_accept();
 
@@ -2372,6 +2342,8 @@ static void init_curses(void)
 	if (!getenv("ESCDELAY")) {
 		set_escdelay(default_esc_delay);
 	}
+
+	update_window_size();
 }
 
 static void init_all(void)
